@@ -2,6 +2,32 @@
 
 extern struct uwsgi_server uwsgi;
 
+void uwsgi_update_load_counters() {
+
+	int i;
+	uint64_t busy_workers = 0;
+	uint64_t idle_workers = 0;
+
+        for (i = 1; i <= uwsgi.numproc; i++) {
+                if (uwsgi.workers[i].cheaped == 0 && uwsgi.workers[i].pid > 0) {
+                        if (uwsgi_worker_is_busy(i) == 0) {
+				idle_workers++;
+			}
+			else {
+				busy_workers++;
+			}
+                }
+        }
+
+	if (busy_workers >= (uint64_t) uwsgi.numproc) {
+		ushared->overloaded++;
+	}
+
+	ushared->busy_workers = busy_workers;
+	ushared->idle_workers = idle_workers;
+
+}
+
 void uwsgi_restore_auto_snapshot(int signum) {
 
 	if (uwsgi.workers[1].snapshot > 0) {
@@ -32,24 +58,25 @@ void uwsgi_unblock_signal(int signum) {
 }
 
 void uwsgi_master_manage_udp(int udp_fd) {
+	char buf[4096];
 	struct sockaddr_in udp_client;
 	char udp_client_addr[16];
 	int i;
 
 	socklen_t udp_len = sizeof(udp_client);
-	ssize_t rlen = recvfrom(udp_fd, uwsgi.wsgi_req->buffer, uwsgi.buffer_size, 0, (struct sockaddr *) &udp_client, &udp_len);
+	ssize_t rlen = recvfrom(udp_fd, buf, 4096, 0, (struct sockaddr *) &udp_client, &udp_len);
 
 	if (rlen < 0) {
-		uwsgi_error("recvfrom()");
+		uwsgi_error("uwsgi_master_manage_udp()/recvfrom()");
 	}
 	else if (rlen > 0) {
 
 		memset(udp_client_addr, 0, 16);
 		if (inet_ntop(AF_INET, &udp_client.sin_addr.s_addr, udp_client_addr, 16)) {
-			if (uwsgi.wsgi_req->buffer[0] == UWSGI_MODIFIER_MULTICAST_ANNOUNCE) {
+			if (buf[0] == UWSGI_MODIFIER_MULTICAST_ANNOUNCE) {
 			}
-			else if (uwsgi.wsgi_req->buffer[0] == 0x30 && uwsgi.snmp) {
-				manage_snmp(udp_fd, (uint8_t *) uwsgi.wsgi_req->buffer, rlen, &udp_client);
+			else if (buf[0] == 0x30 && uwsgi.snmp) {
+				manage_snmp(udp_fd, (uint8_t *) buf, rlen, &udp_client);
 			}
 			else {
 
@@ -57,7 +84,7 @@ void uwsgi_master_manage_udp(int udp_fd) {
 				int udp_managed = 0;
 				for (i = 0; i < 256; i++) {
 					if (uwsgi.p[i]->manage_udp) {
-						if (uwsgi.p[i]->manage_udp(udp_client_addr, udp_client.sin_port, uwsgi.wsgi_req->buffer, rlen)) {
+						if (uwsgi.p[i]->manage_udp(udp_client_addr, udp_client.sin_port, buf, rlen)) {
 							udp_managed = 1;
 							break;
 						}
@@ -66,12 +93,12 @@ void uwsgi_master_manage_udp(int udp_fd) {
 
 				// else a simple udp logger
 				if (!udp_managed) {
-					uwsgi_log("[udp:%s:%d] %.*s", udp_client_addr, ntohs(udp_client.sin_port), (int) rlen, uwsgi.wsgi_req->buffer);
+					uwsgi_log("[udp:%s:%d] %.*s", udp_client_addr, ntohs(udp_client.sin_port), (int) rlen, buf);
 				}
 			}
 		}
 		else {
-			uwsgi_error("inet_ntop()");
+			uwsgi_error("uwsgi_master_manage_udp()/inet_ntop()");
 		}
 
 	}
@@ -346,6 +373,7 @@ int master_loop(char **argv, char **environ) {
 #ifdef UWSGI_SSL
 	uwsgi_start_legions();
 #endif
+	uwsgi_metrics_start_collector();
 
 	uwsgi_add_reload_fds();
 
@@ -583,8 +611,10 @@ int master_loop(char **argv, char **environ) {
 
 			/* all processes ok, doing status scan after N seconds */
 			check_interval = uwsgi.shared->options[UWSGI_OPTION_MASTER_INTERVAL];
-			if (!check_interval)
+			if (!check_interval) {
 				check_interval = 1;
+				uwsgi.shared->options[UWSGI_OPTION_MASTER_INTERVAL] = 1;
+			}
 
 
 			// add unregistered file monitors
@@ -619,14 +649,14 @@ int master_loop(char **argv, char **environ) {
 
 			if (ushared->rb_timers_cnt > 0) {
 				min_timeout = uwsgi_min_rb_timer(rb_timers, NULL);
-				if (min_timeout == NULL) {
-					check_interval = uwsgi.shared->options[UWSGI_OPTION_MASTER_INTERVAL];
-				}
-				else {
-					check_interval = min_timeout->value - uwsgi_now();
-					if (check_interval <= 0) {
+				if (min_timeout) {
+					int delta = min_timeout->value - uwsgi_now();
+					if (delta <= 0) {
 						expire_rb_timeouts(rb_timers);
-						check_interval = 0;
+					}
+					// if the timer expires before the check_interval, override it
+					else if (delta < check_interval) {
+						check_interval = delta;
 					}
 				}
 			}
@@ -640,6 +670,9 @@ int master_loop(char **argv, char **environ) {
 				}
 			}
 
+			// update load counter
+			uwsgi_update_load_counters();
+
 
 			// check uwsgi-cron table
 			if (ushared->cron_cnt) {
@@ -680,8 +713,10 @@ int master_loop(char **argv, char **environ) {
 			uwsgi_master_check_idle();
 
 			check_interval = uwsgi.shared->options[UWSGI_OPTION_MASTER_INTERVAL];
-			if (!check_interval)
+			if (!check_interval) {
 				check_interval = 1;
+				uwsgi.shared->options[UWSGI_OPTION_MASTER_INTERVAL] = 1;
+			}
 
 
 			// get listen_queue status
