@@ -1,4 +1,4 @@
-#include "../../uwsgi.h"
+#include <uwsgi.h>
 #include <Python.h>
 
 #include <frameobject.h>
@@ -180,6 +180,11 @@ struct uwsgi_python {
 	int start_response_nodelay;
 
 	char *programname;
+	int wsgi_strict;
+	int wsgi_accept_buffer;
+
+	char *raw;
+	PyObject *raw_callable;
 };
 
 
@@ -276,6 +281,10 @@ struct uwsgi_buffer *uwsgi_python_exception_repr(struct wsgi_request *);
 struct uwsgi_buffer *uwsgi_python_backtrace(struct wsgi_request *);
 void uwsgi_python_exception_log(struct wsgi_request *);
 
+int uwsgi_python_send_body(struct wsgi_request *, PyObject *);
+
+int uwsgi_request_python_raw(struct wsgi_request *);
+
 #define py_current_wsgi_req() current_wsgi_req();\
 			if (!wsgi_req) {\
 				return PyErr_Format(PyExc_SystemError, "you can call uwsgi api function only from the main callable");\
