@@ -1,6 +1,6 @@
 # uWSGI build system
 
-uwsgi_version = '1.9.18.2'
+uwsgi_version = '1.9.19'
 
 import os
 import re
@@ -538,6 +538,7 @@ class uConf(object):
             'core/setup_utils', 'core/clock', 'core/init', 'core/buffer', 'core/reader', 'core/writer', 'core/alarm', 'core/cron', 'core/hooks',
             'core/plugins', 'core/lock', 'core/cache', 'core/daemons', 'core/errors', 'core/hash', 'core/master_events', 'core/chunked',
             'core/queue', 'core/event', 'core/signal', 'core/strings', 'core/progress', 'core/timebomb', 'core/ini', 'core/fsmon', 'core/mount',
+            'core/metrics',
             'core/rpc', 'core/gateway', 'core/loop', 'core/cookie', 'core/querystring', 'core/rb_timers', 'core/transformations', 'core/uwsgi']
         # add protocols
         self.gcc_list.append('proto/base')
@@ -545,6 +546,7 @@ class uConf(object):
         self.gcc_list.append('proto/http')
         self.gcc_list.append('proto/fastcgi')
         self.gcc_list.append('proto/scgi')
+        self.gcc_list.append('proto/puwsgi')
         self.include_path = []
 
         if 'UWSGI_INCLUDES' in os.environ:
